// Emacs style mode select   -*- C++ -*- 
//-----------------------------------------------------------------------------
//
// $Id:$
//
// Copyright (C) 1993-1996 by id Software, Inc.
//
// This source is available for distribution and/or modification
// only under the terms of the DOOM Source Code License as
// published by id Software. All rights reserved.
//
// The source is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// FITNESS FOR A PARTICULAR PURPOSE. See the DOOM Source Code License
// for more details.
//
// $Log:$
//
// DESCRIPTION:
//	DOOM graphics stuff for GGI ( http://www.ggi-project.org)
//	Maintainer: Uwe Maurer - uwe_maurer@t-online.de
//			http://home.t-online.de/home/uwe_maurer
//-----------------------------------------------------------------------------

static const char
rcsid[] = "$Id: i_x.c,v 1.6 1997/02/03 22:45:10 b1 Exp $";

#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <ggi/ggi.h>
#include <ggi/keyboard.h>

#include "doomstat.h"
#include "i_system.h"
#include "v_video.h"
#include "m_argv.h"
#include "d_main.h"
#include "r_draw.h"

#include "doomdef.h"

static int     lastmousex = 0;
static int     lastmousey = 0;
boolean                mousemoved = false;
int buttonstate=0;

static ggi_visual_t 	ggivis;

static int	width;
static int	height;
static int 	db;
static int	ypos;
static int 	virt_y;

// #define KEY_DEBUG

#ifdef KEY_DEBUG
char str[32];
char * bits(int a)
{
	int i;
	for (i=0;i<8;i++) str[16-i]=((a & (1<<i))!=0)+'0';	
	str[8]=' ';
	for (i=8;i<16;i++) str[15-i]=((a & (1<<i))!=0)+'0';	
	return str;
}
#endif

int key(int sym)
{
    	int rc=0;
#ifdef KEY_DEBUG
	char text[100];
	ggiSetGCForeground(ggivis,0);
	ggiDrawBox(ggivis,0,0,320,24);
	ggiSetGCForeground(ggivis,255);
	sprintf(text," %c",KVAL(U(sym)));
	ggiPuts(ggivis,0,0,text);
	sprintf(text," %i %x %s",U(sym),U(sym),bits(U(sym)));
	ggiPuts(ggivis,0,8,text);
	sprintf(text," %i %x %s",code,code,bits(code));
	ggiPuts(ggivis,0,16,text);
#endif
	switch(GII_KTYP(sym))
	{
		case GII_KT_LATIN1:
		switch(GII_KVAL(sym))
		{
			case '\033':	rc = KEY_ESCAPE;	break;
			case '\011':	rc = KEY_TAB;		break;
			
			default:
				rc=GII_KVAL(sym);
				if (rc >='A' && rc <='Z')
					rc = rc - 'A' + 'a';
				
		}
		break;

		case 0xf2:	// ALT + SHIFT + ARROW KEY
/*			switch(code)
			{
				case 0x69: rc=KEY_LEFTARROW;	break;
				case 0x6a: rc=KEY_RIGHTARROW;	break;
				case 0x67: rc=KEY_UPARROW;	break;
				case 0x6c: rc=KEY_DOWNARROW;	break;
				default:
				printf("Unknown special key (%i)!\n",code);
			}
			break;
*/		default:
		switch(sym)
		{
			 
      			case GIIK_Up:	rc = KEY_UPARROW;	break;
      			case GIIK_Down:	rc = KEY_DOWNARROW;	break;
      			case GIIK_Left:	rc = KEY_LEFTARROW;	break;
			case GIIK_Right:	rc = KEY_RIGHTARROW;	break;
			case GIIK_Enter:	rc = KEY_ENTER;		break;
			case GIIK_Ctrl: rc=KEY_RCTRL; 	break;
			case GIIK_Shift: rc=KEY_RSHIFT;	break;

			case GIIK_AltGr:
			case GIIK_Alt: rc=KEY_RALT;		break;
      			case GIIK_F1:	rc = KEY_F1;		break;
			case GIIK_F2:	rc = KEY_F2;		break;
			case GIIK_F3:	rc = KEY_F3;		break;
			case GIIK_F4:	rc = KEY_F4;		break;
			case GIIK_F5:	rc = KEY_F5;		break;
			case GIIK_F6:	rc = KEY_F6;		break;
			case GIIK_F7:	rc = KEY_F7;		break;
			case GIIK_F8:	rc = KEY_F8;		break;
			case GIIK_F9:	rc = KEY_F9;		break;
			case GIIK_F10:	rc = KEY_F10;		break;
			case GIIK_F11:	rc = KEY_F11;		break;
			case GIIK_F12:	rc = KEY_F12;		break;
      			case GIIK_Pause:	rc = KEY_PAUSE;		break;
	
		}
	}

    return rc;

}

void I_ShutdownGraphics(void)
{
	ggiClose(ggivis);
	ggiExit();
}


void I_StartFrame (void)
{

}

void I_GetEvent(void)
{
	event_t event;
	ggi_event ev;
	int evmask=emKeyPress|emKeyRelease|emPtrButtonPress|emPtrButtonRelease|emPtrMove;

	while (1)
	{
		struct timeval t={0,0};
		if (ggiEventPoll(ggivis,evmask,&t)==0) return;

		ggiEventRead(ggivis,&ev,evmask);

		switch (ev.any.type)
		{
			case evKeyPress:
				event.type = ev_keydown;
				event.data1 = key(ev.key.sym);
				D_PostEvent(&event);
				break;
		 	case evKeyRelease:
				event.type = ev_keyup;
				event.data1 = key(ev.key.sym);
				D_PostEvent(&event);
				break;
			case evPtrButtonPress:
				event.type = ev_mouse;
				buttonstate = event.data1 = buttonstate | ev.pbutton.button;
				event.data2 = event.data3 = 0;
				D_PostEvent(&event);
				break;
			case evPtrButtonRelease:
				event.type = ev_mouse;
				buttonstate = event.data1 = buttonstate ^ ev.pbutton.button;
				event.data2 = event.data3 = 0;
				D_PostEvent(&event);
				break;
			case evPtrAbsolute:
				event.type = ev_mouse;
				event.data1 = buttonstate;
				event.data2 = (ev.pmove.x - lastmousex) << 2;
				event.data3 = (lastmousey - ev.pmove.y) << 2;

				if (event.data2 || event.data3)
				{
					lastmousex = ev.pmove.x;
					lastmousey = ev.pmove.y;
					if (ev.pmove.x != screenwidth/2 &&
						ev.pmove.y != screenheight/2)
					{
						D_PostEvent(&event);
						mousemoved = false;
					} else
					{
						mousemoved = true;
					}
				}
				break;
			case evPtrRelative:
				event.type = ev_mouse;
				event.data1 = buttonstate;
				event.data2 = ev.pmove.x << 2;
				event.data3 = -ev.pmove.y << 2;

				if (event.data2 || event.data3)
				{
					lastmousex += ev.pmove.x;
					lastmousey += ev.pmove.y;
					if (ev.pmove.x != screenwidth/2 &&
						ev.pmove.y != screenheight/2)
					{
						D_PostEvent(&event);
						mousemoved = false;
					} else {
						mousemoved = true;
					}
				}
				break;
			
	    }
	}
}

void I_StartTic (void)
{
	/* only call if GGI is initialized
	 * Fixes crash in IPX network game.
	 */
	if (ggivis) {
		I_GetEvent();
	}
}


void I_UpdateNoBlit (void)
{
}

void I_FinishUpdate (void)
{

    static int	lasttic;
    int		tics;
    int		i;

    // draws little dots on the bottom of the screen
    if (devparm)
    {
	i = I_GetTime();
	tics = i - lasttic;
	lasttic = i;
	if (tics > 20) tics = 20;

	for (i=0 ; i<tics*2 ; i+=2)
	    screens[0][ (screenheight-1)*screenwidth+ i] = 0xff;
	for ( ; i<20*2 ; i+=2)
	    screens[0][ (screenheight-1)*screenwidth + i] = 0x0;
    
    }
	if (!db)
	{
		ggiPutBox(ggivis,0,0,width,height,screens[0]);
	}
	else
	{
		ggiSetOrigin(ggivis,0,ypos);
		ypos+=height;
		if (ypos>=virt_y) ypos=0;
//		screens[0]=ggiGetInfo(ggivis)->fb.linear+width*ypos;
		screens[0]=ggiDBGetBuffer(ggivis,0)->write+width*ypos;
	}

	ggiFlush(ggivis);
	I_GetEvent();
}


void I_SetPalette (byte* pal)
{
	int i;
	ggi_color col[256];
	byte *gamma=gammatable[usegamma];

	for (i=0;i<256;i++) 
	{
		col[i].r=gamma[(*pal++)] << (GGI_COLOR_PRECISION-8);
		col[i].g=gamma[(*pal++)] << (GGI_COLOR_PRECISION-8);
		col[i].b=gamma[(*pal++)] << (GGI_COLOR_PRECISION-8);
	}
	ggiSetPalette(ggivis,0,256,col);
	ggiFlush(ggivis);
}

void InitGraphLib(void)
{
	/* dummy here */
}

void I_InitGraphics(void)
{
	ggiInit();
	ggivis=ggiOpen(NULL);

	signal(SIGINT, (void (*)(int)) I_Quit);

	db = M_CheckParm("-doublebuffer") > 0;
	width = screenwidth;
	height = screenheight;

	virt_y = db ? 2*height : height;

	if (ggiSetGraphMode(ggivis,width,height,width,virt_y,GT_8BIT)<0)
		I_Error("Can't set mode %ix%i %ix%i\n",
			width,height,width,virt_y);
	
	if (db)
	{
		printf("db\n");
//			screens[0]=ggiGetInfo(ggivis)->fb.linear;
			screens[0]=ggiDBGetBuffer(ggivis,0)->write;
		printf("db %p\n",screens[0]);
			if (!screens[0])
				I_Error("No linear frame buffer!\n");
	}

	ggiAddFlags(ggivis,GGIFLAG_ASYNC);

}

